c
c     This file is part of the Test Set for BVP solvers
c     http://www.dm.uniba.it/~bvpsolvers/
c
c        Problem Shock Wave, bvpT24
c        ODE of dimension 2
c
c     DISCLAIMER: see
c     http://www.dm.uniba.it/~bvpsolvers/testsetbvpsolvers/?page_id=34
c
c     The most recent version of this source file can be found at
c     http://www.dm.uniba.it/~bvpsolvers/bvpTestSet/fortransrc/problems/bvpT24.f
c
c
c
c-----------------------------------------------------------------------
c
      integer function pidate()
      pidate = 20130531
      return 
      end
c-----------------------------------------------------------------------
      subroutine prob(fullnm,problm,type,
     +                neqn,aleft,aright,nlbc, ms,
     +                numjac,numbcjac,linear,rpar,ipar)
      implicit none
      character*(25), intent(out) :: fullnm
      character*(15), intent(out) ::problm
      character*(5), intent(out) :: type
      integer, intent(out) ::  neqn,ipar(1), nlbc, ms(2)
      double precision, intent(out) :: aleft, aright, rpar(2)
      logical, intent(out) :: numjac, numbcjac,linear

      double precision :: EPS
      common /pars/ EPS
c
c  For Singularly perturbed BVP (SPBVP) eps should be the
c first elements of parms and the first element of rpar
c

      fullnm = ' '
      problm = ' '
      type   = ' '
      fullnm = 'Problem bvpT24'
      problm = 'bvpT24'
      type   = 'SPBVP'
      ms(1)  = 1
      ms(2)  = 1
      neqn = 2
      nlbc = 1
      aleft    = 0.0d0
      aright   = 1.0d0
      numjac   = .false.
      numbcjac = .false.
      linear   = .false.

c
c      EPS is an input parameter in report.f
c
c rpar(1) may contains epsin, the starting value for the ontinuation codes
c rpar(1) = 0 means that epsin is the default value
      rpar(1)  =  0.5d0
c for this problem ipar is a dummy variable
      ipar(1)  = 0
      return
      end
c
c-----------------------------------------------------------------
c
      subroutine settolerances(neqn,rtol,atol,tolvec,ntol,ltol)
      integer, intent(in) :: neqn
      integer, intent(out) :: ntol, ltol(*)
      logical, intent(out) :: tolvec
      double precision, intent(in out) :: rtol(neqn), atol(neqn)

      tolvec  = .true.
      ntol = neqn
      ltol(1)=1

      DO i=2,ntol
         ltol(i) = i
         rtol(i) = rtol(1)
         atol(i) = atol(1)
      ENDDO
      

      return
      end
c---------------------------------------------------------------------
      subroutine setoutput(neqn,solref,printsolout,
     +                    nindsol,indsol)

      logical, intent(out) :: solref, printsolout
      integer, intent(in) :: neqn
      integer, intent(out) ::  nindsol
      integer, intent(out) :: indsol(*)

c the reference solution is available
      solref = .false.

c output file is required
      printsolout = .true.

c default values if printsolout is .true.
      nindsol = neqn
c only nindsol component of indsol are referenced
      do i=1,nindsol
          indsol(i) = i
      end do  


      return
      end
c-----------------------------------------------------
c Initialiser for parameter common block
c used only for the R interface
c
c
      SUBROUTINE initrparms(bvpparms)
      EXTERNAL bvpparms
c
c For Singularly Perturbed BVP (SPBVP) eps should be the
c first elements of parms and the first element of rpar
c
      DOUBLE PRECISION parms(1)
      COMMON / pars / parms

      CALL bvpparms(1, parms)

      RETURN
      END



c-----------------------------------------------------------------------
      subroutine init(neqn,aleft,aright,ms,yval0, nmsh, givmsh,
     +        givey,xguess,yguess,dmguess,nudim)
      integer, intent(in) :: neqn,nudim,ms(neqn)
      double precision, intent(in) :: aleft,aright
      integer, intent(out) :: nmsh
      double precision, intent(out) :: xguess(*),yguess(nudim,*)
      double precision, intent(out) :: dmguess(nudim,*)
      double precision, intent(out) :: yval0(1:neqn)
      logical, intent(out) :: givmsh, givey
      double precision h

      givmsh = .true.
      givey = .true.
      yval0(1:neqn) = 0.5d0
      nmsh = 3*5+1

c---- dummy values, not used if givmsh=FALSE, givey=false

      h =(aright-aleft)/(nmsh-1)
      xguess(1) = aleft
      do i=2,nmsh
         xguess(i)=xguess(i-1)+h
      end do
      xguess(nmsh)= aright

      ij = 1
      DO i=1,neqn
        do j=1,ms(i)
           yguess(ij,1:nmsh) = 0.5d0
           ij = ij+1
        end do
      end do

      DO i=1,neqn
           dmguess(i,1:nmsh) = 0.5d0
      end do
      return
      end

c-----------------------------------------------------------------------
      subroutine feval(neqn,x,z,f,rpar,ipar)
      integer, intent(in) :: neqn,ipar(*)
      double precision, intent(in):: x,z(1:neqn)
      double precision, intent(inout):: rpar(*)
      double precision, intent(out):: f(1:neqn)
      DOUBLE PRECISION EPS,PI,PIX,AX,GA,APX

      common /pars/ EPS

      AX=1.D0+X**2
      APX=2.D0*X
      GA=1.4D0
      F(1)=Z(2)
      F(2)=(((1.D0+GA)/2.D0-EPS*APX)*Z(1)*Z(2)-Z(2)/Z(1)-
     *          (APX/AX)*(1.D0-(GA-1.D0)*Z(1)**2/2.D0))/(EPS*AX*Z(1))


      return
      end
c-----------------------------------------------------------------------
c      subroutine jeval(neqn,x,y,dfdy,rpar,ipar)
c      integer, intent(in) ::  neqn,ipar(*)
c      double precision, intent(in):: x,y(1:neqn),rpar(*)
c      double precision, intent(out):: dfdy(1:neqn,1:neqn)

c      double precision :: EPS
c      common /pars/ EPS


c      dfdy(1,1) = 0d0
c      dfdy(1,2) = 1d0
c      dfdy(2,1) = 1d0/EPS
c      dfdy(2,2) = 0d0

c      return
c      end

c-----------------------------------------------------------------------
      subroutine bceval(i,neqn,z,bc,rpar,ipar)
      integer, INTENT(IN) :: i, neqn, ipar(*)
      double precision, INTENT(IN) :: z(neqn), rpar(*)
      double precision, INTENT(OUT) :: bc
      double precision :: EPS
      common /pars/ EPS



      IF (I.EQ.1) bc=Z(1)-0.9129D0
      IF (I.EQ.2) bc=Z(1)-0.375D0

      return
      end

      subroutine dbceval(i, ncomp,y, dbc, rpar,ipar)
      integer, intent(in) ::  i,ncomp, ipar(*)
      double precision, INTENT(IN) ::  y(ncomp), rpar(*)
      double precision, INTENT(OUT) :: dbc(ncomp)
      double precision :: EPS
      common /pars/ EPS

      dbc(1)=1.D0
      dbc(2)=0.D0

      return
      end

c
c----------------------------------------------------------------------
c  Numerical Jacobian
c
C     SUBROUTINE  jeval(R, X, Z, JF0, RPAR,IPAR)
c      SUBROUTINE  fnumjac(R, X0, Y0, JF0, RPAR,IPAR)
      SUBROUTINE jeval(NCOMP,X,Z,DF,RPAR,IPAR)
      IMPLICIT NONE
      INTEGER NCOMP, IPAR, I, J
      DOUBLE PRECISION X, Z, DF, RPAR
      DIMENSION Z(*),DF(NCOMP,*)
      DIMENSION RPAR(*), IPAR(*)

      DOUBLE PRECISION EPS,PI,PIX,AX,GA,APX
      COMMON / pars / EPS,PI
c      EPS   = RPAR(1)
c      PI    = RPAR(2)
      AX=1.D0+X**2
      APX=2.D0*X
      GA=1.4D0
      DF(1,1)=0.0D0
      DF(1,2)=1.0D0
      DF(2,1)=((2.D0*Z(2))/Z(1)**3+APX/(AX*Z(1)**2)+
     *            (APX*(GA-1.D0))/(2.D0*AX))/(EPS*AX)
      DF(2,2)=(((1.D0+GA)/2.D0-EPS*APX)*Z(1)-1.D0/Z(1))/(EPS*AX*Z(1))
      RETURN
      END


c-----------------------------------------------------------------------
      subroutine solut(neqn,x,y,nmsh,rpar,ipar)
      integer, intent(in) ::  neqn, ipar(*)
      double precision, intent(in) :: x(nmsh),rpar(*)
      double precision, intent(out) :: y(1:neqn,1:nmsh)

      double precision :: lambda, lambda2
      common /pars/ lambda




      return
      end
