## ========================================================================
##  
## ========================================================================
bvpRtest <- function(problem="bvpT1", tol=NULL, eps=NULL,
                     solver = c("twpbvp_m","twpbvp_l",
                                    "twpbvpc_l", "twpbvpc_m","acdc","acdcc",
                                    "colmod","colnew", "colsys",
                                    "bvpshoot"),
                     printall=TRUE,nmax=10000,...){ 
  
 #  solver is one of "twpbvp_l","twpbvp_m","twpbvpc_l",
 #  "twpbvpc_m","acdc","acdcc","colmod","colnew", "colsys", "bvpshoot"
 
  if (is.null(tol)) {
    stop("bvpRtest : gives an input tolerance ")
  }
  

  if ( length(solver) > 1){
    cat('solver should be one of :\n')
    cat(solver)
    stop("bvpRtest : choose one solver")
  }
  
  listsolver = c("twpbvp_m","twpbvp_l","twpbvp_m",
                     "twpbvpc_l", "twpbvpc_m","acdc","acdcc",
                     "colmod","colnew", "colsys",
                     "bvpshoot")
  OKI = FALSE
  for (solverE in listsolver){ 
     OKI = OKI || (solver == solverE)}
  
  
  if ( ! OKI ){
    cat('solver should be one of :\n')
    cat(listsolver)
    stop("bvpRtest : choose one solver")
  }
  require(bvpSolve)
  
  dll = !is.function(problem)
  
  #  
  # -----  solution of the problem written in fortran using the standard
  # 
  
  if (dll) {
    if (solver == "bvpshoot"){
      stop(sprintf("bvpRtest : bvpshoot cannot solve dll problems"))
      solver = "twpbvp_m"
      solver = bvptwp
      lobatto=FALSE
      cond=FALSE
    }
    
    
  # fortran function in input
  problist<-system(sprintf("ls ../fortransrc/problems/%s%s",problem,".f"),show.output.on.console=FALSE,intern=TRUE)
  probRlist<-system(sprintf("ls ../R/%s%s",problem,".R"),show.output.on.console=FALSE,intern=TRUE) 
  pldll <- .Platform$dynlib.ext
  probdll<-system(sprintf("ls  ../fortransrc/problems/%s%s",problem,pldll),show.output.on.console=FALSE,intern=TRUE,ignore.stderr=TRUE)  
  dllname <- sprintf("../fortransrc/problems/%s%s",problem,pldll)
 
    
  if (is.null(attr(problist,"status")) && !(is.null(attr(probdll,"status"))) )
      {probcompile<-sprintf("R CMD SHLIB ../fortransrc/problems/%s%s",problem,".f")
      system(probcompile)
      dyn.load(dllname)
      }
  else if (is.null(attr(probdll,"status"))){
      dyn.load(dllname) 
      }
  else { 
      stop(sprintf("bvpRtest : no problem named %s ",problem))
      }
      
   
  if (is.loaded("prob",PACKAGE=problem )){  
       fullnm <- character(len=1)
       problm <- character(len=1)
       typebvp  <- character(len=1)
       neqn   <- 0
       aleft  <- 0.0
       aright <- 0.0
       nlbc   <- 0
       numjac <- TRUE
       numbcjac <- TRUE
       linear <- TRUE
       rpar   <- rep(0,10)
       ipar   <- rep(0,10)
       ms     <- rep(0,10)
       outprob <- .Fortran("prob",fullnm=fullnm,problm=problm,typebvp=typebvp,
                    neqn=as.integer(neqn),aleft=as.double(aleft),aright=as.double(aright),nlbc=as.integer(nlbc),ms=as.integer(ms),
                    numjac=as.logical(numjac),numbcjac=as.logical(numbcjac),linear=as.logical(linear),rpar=as.double(rpar),
                    ipar=as.integer(ipar), PACKAGE=problem) 
       fullnm <- substr(outprob$fullnm,1,20)
       problm <- substr(outprob$problm,1,12)
       typebvp <- substr(outprob$typebvp,1,5)
       neqn <- outprob$neqn
       nlbc <- outprob$nlbc
       ms  <- outprob$ms[1:neqn]
       nstar   <- sum(ms[1:neqn])
       aleft   <- outprob$aleft
       aright  <- outprob$aright
       numjac <- outprob$numjac
       numbcjac <- outprob$numbcjac
       linear <- outprob$linear
       rpar  <- outprob$rpar  
       ipar  <- outprob$ipar
       
       }
  else
   {
    stop(sprintf("bvpRtest : no function named prob in %s",problem))
    }
   
  if (grep("SPBVP",typebvp) > 0) {
    if (is.null(eps)) eps= 1e-1
       }
   else { 
      if ( (solver == "acdc") || (solver == "acdcc") || (solver = "colmod")) {
          stop(sprintf("bvpRtest : A continuation solver cannot solve %s problems ",typebvp))
         }}
    
  if (is.loaded("settolerances",PACKAGE=problem)) {
      rtol<-rep(0,nstar)
      rtol[1] <- tol
      atol<-rep(0,nstar)
      atol[1]  <- tol
      tolvec <- TRUE
      ntol <- 0
      ltol <- rep(0,nstar)               
      outtol<- .Fortran("settolerances",neqn=as.integer(nstar),rtol=as.double(rtol),atol=as.double(atol),tolvec=as.logical(tolvec),ntol=as.integer(ntol), ltol=as.integer(ltol),PACKAGE=problem)
      rtol <- outtol$rtol
      atol <- outtol$atol
      tolvec <- outtol$tolvec
      ntol <- outtol$ntol
      ltol <- outtol$ntol    
   
    }
    else
    {
      stop(sprintf("bvpRtest : no function named settolerances in %s",problem))
      }
    if (is.loaded("init",PACKAGE=problem)){ 
      yval0 <- rep(0,nstar)
      nmax <- 10000
      nmsh <- nmax
      givmsh <- FALSE
      givey <- FALSE
    
      xguess <- rep(0,nmsh)
      yguess <- NULL
      dmguess <- NULL
      for (i in 1:nstar){
        yguess <- rbind(yguess,rep(0,nmsh))
        dmguess <- rbind(dmguess,rep(0,nmsh))
      }
      nudim <- nstar
      
      outinit <- .Fortran("init",neqn=as.integer(neqn), ms=as.integer(ms),yval0=as.double(yval0),
            nmsh=as.integer(nmsh), givmsh=as.logical(givmsh),givey=as.logical(givey),
                          xguess=as.double(xguess),yguess=as.double(yguess),
                          dmguess=as.double(dmguess), nudim=as.integer(nudim),PACKAGE=problem)
      yval0   <- outinit$yval0
      givmsh  <- outinit$givmsh
      givey   <- outinit$givey
      nmsh    <- outinit$nmsh
      xguess  <- outinit$xguess[1:nmsh] 
      nstar   <- sum(ms[1:neqn])
      yguess  <- outinit$yguess[1:(nmsh*nstar)]    
      yguess  <- matrix(yguess, nrow=nstar,ncol=nmsh)
      dmguess <- outinit$dmguess[1:(nmsh*neqn)]    
      dmguess <- matrix(dmguess, nrow=neqn,ncol=nmsh)      
     }
    else
    {
       stop(sprintf("bvpRtest : no function named init in %s",problem))
    }
      
    if (givmsh)
      x <-xguess
    else
      x <- seq(aleft,aright, by = (aright-aleft)/(nmsh-1) )
  
    
    if (!givey){ 
       yguess <- NULL
       xguess <- NULL}
    
    
    parms = c(EPS=eps)
    
    switch(solver,
           "twpbvp_l"=, "twpbvp_m"=,"acdc" = cond <- FALSE,
           "twpbvpc_l"=, "twpbvpc_m"= ,"acdcc"= cond <- TRUE
          )
    
    switch(solver,
           "twpbvp_l"=, "twpbvpc_l" = lobatto <- TRUE,
           "twpbvp_m"=, "twpbvpc_m" = lobatto <- FALSE
         )
    
    switch(solver, 
            "colmod"=,"colnew" = bspline <- FALSE,
            "colsys" = bspline <- TRUE)
    
    # define zeta for colsys, colnew, colmod
    
    posbound = rep(aleft,nlbc)
    posbound = c(posbound, rep(aright,nstar-nlbc))
    
    
    runTime<-system.time(switch(solver, 
        "twpbvp_l"=, "twpbvpc_l"=,  "twpbvp_m"=, "twpbvpc_m" = 
                   sol<- bvptwp(x=x, leftbc = nlbc, order=ms,xguess=xguess,yguess=yguess,
                          func = "feval", ncomp = neqn,parms=parms, rpar=rpar, ipar=ipar,islin=linear,
                          initfunc="initrparms",bound = "bceval", jacfunc = "jeval", jacbound = "dbceval",
                          dllname=problem,atol=rtol,lobatto=lobatto,cond=cond,nmax=nmax,...),
        "acdc"=, "acdcc" = sol<- bvptwp(x=x, leftbc = nlbc,order=ms, xguess=xguess,yguess=yguess,
                                 eps=eps,epsini=rpar[1], func = "feval", ncomp = neqn,parms=parms, rpar=rpar, ipar=ipar,islin=linear,
                                 initfunc="initrparms",bound = "bceval", jacfunc = "jeval", jacbound = "dbceval",
                                dllname=problem,atol=rtol,lobatto=lobatto,cond=cond,nmax=nmax,...),    
        "colmod" = sol<- bvpcol(x=x,posbound=posbound, leftbc = nlbc, order=ms,xguess=xguess,yguess=yguess,
                                 eps=eps,epsini=rpar[1], func = "feval", ncomp = neqn,parms=parms, rpar=rpar, ipar=ipar,islin=linear,
                                 initfunc="initrparms",bound = "bceval", jacfunc = "jeval", jacbound = "dbceval",
                                 dllname=problem,atol=rtol,nmax=nmax, ...),
        "colnew"=,"colsys" = sol<- bvpcol(x=x, posbound=posbound, leftbc = nlbc,order=ms, xguess=xguess,yguess=yguess,
                                  func = "feval", ncomp = neqn,parms=parms, rpar=rpar, ipar=ipar,islin=linear,
                                 initfunc="initrparms",bound = "bceval", jacfunc = "jeval", jacbound = "dbceval",
                                 dllname=problem,atol=rtol,bspline=bspline, nmax=nmax,...)
                                
                                )
    )
    idid <- attributes(sol)$istate[1]
    
    
    if (attributes(sol)$name == "bvpcol") { 
      if (idid !=1) { 
        warning("bvpRtest : the code has encountered trouble in the computation of the 
                 solution")
        mescd = NA 
        solt = NA
        runTime[3] = NA
        return(list(sol=sol,solt=solt,runTime=runTime[3],mescd=mescd))  
      }}
    else if (attributes(sol)$name == "bvptwp") 
    { 
      if (idid > 0 | (idid < 0 &  (solver=="acdc" | solver =="acdcc")) ){ 
        warning("bvpRtest : the code has encountered trouble in the computation of the 
                solution")
        mescd = NA 
        solt = NA
        runTime[3] = NA
        return(list(sol=sol,solt=solt,runTime=runTime[3],mescd=mescd))  
      }}
    if ( (solver == "colnew") | (solver == "colmod") | (solver == "colsys")){
      ninter = attributes(sol)$istate[7]
      x= attributes(sol)$rstate[1:(ninter+1)]    
      solp <- matrix(nrow=ninter+1,ncol=nstar+1)
      solp <- approx(sol,x)  
      
      }
    else
    { x = sol[,1]
      solp <- sol}
    
   
      
    
    if ( is.loaded("setoutput",PACKAGE=problem))  {
      solref <-  FALSE
      printsolout <-FALSE
      nindsol <-neqn
      indsol <-rep(0,neqn)
      outset <- .Fortran("setoutput",neqn=as.integer(neqn), solref=as.logical(solref),
                         prinsolout=as.logical(printsolout),nindsol=as.integer(nindsol),
                         indsol=as.integer(indsol),PACKAGE=problem)
       solref  <- outset$solref
       nindsol <- outset$nindsol
       indsol  <- outset$indsol[1:nindsol]
    }
    else{
      warning(sprintf("bvpRtest : no function named setoutput in %s",problem))
      nindsol <-  neqn
      indsol  <- 1:neqn
    }
   
   
    if (is.loaded("solut",PACKAGE=problem) && solref)  {
        nmsh <- length(solp[,1])
        x = solp[,1]  
        solt <- t(solp[,-1])
        outsol <- .Fortran("solut",nstar=as.integer(nstar), x=as.double(x),
                           solt=as.double(solt),nmsh=as.integer(nmsh),
                           rpar=as.double(rpar),ipar=as.integer(ipar),PACKAGE=problem)
        solt <-  outsol$solt
        print(solt)
        solt <- matrix(solt, ncol=nmsh,nrow=nstar)
        solt <- t(solt) 
        print(x)
        print(solp)
        print(solt)}
    else{
        
        xguess <- solp[,1]
        yguess <- t(solp[,-1])
        nmaxacc = max(length(xguess)*2,nmax)
        
        switch(solver, 
           "twpbvp_l" =, "twpbvpc_l" = ,  "twpbvp_m" =,  "twpbvpc_m" = 
                 solacc<- bvptwp(x=x, leftbc = nlbc, order=ms,xguess=xguess,yguess=yguess,
                           func = "feval", ncomp = neqn,parms=parms, rpar=rpar, ipar=ipar,islin=linear,
                           initfunc="initrparms",bound = "bceval", jacfunc = "jeval", jacbound = "dbceval",
                           dllname=problem,atol=rtol/1000,lobatto=lobatto,cond=cond,allpoint=FALSE,nmax=nmaxacc,...),
           "acdc" =,"acdcc" = solacc<- bvptwp(x=x, leftbc = nlbc,  order=ms,xguess=xguess,yguess=yguess,
                                       eps=eps,epsini=rpar[1], func = "feval", ncomp = neqn,parms=parms, rpar=rpar, ipar=ipar,islin=linear,
                                        initfunc="initrparms",bound = "bceval", jacfunc = "jeval", jacbound = "dbceval",
                                        dllname=problem,atol=rtol/1000,cond=cond,allpoint=FALSE,nmax=nmaxacc,...),
           "colmod" =solacc<- bvpcol(x=x, posbound=posbound,leftbc = nlbc,order=ms, xguess=xguess,yguess=yguess,
                                   eps=eps,epsini=rpar[1], func = "feval", ncomp = neqn,parms=parms, rpar=rpar, ipar=ipar,islin=linear,
                                   initfunc="initrparms",bound = "bceval", jacfunc = "jeval", jacbound = "dbceval",
                                   dllname=problem,atol=rtol/1000,nmax=nmaxacc,...),
            "colnew" =, "colsys" = solacc<- bvpcol(x=x, posbound=posbound,leftbc = nlbc,  order=ms,xguess=xguess,yguess=yguess,
                                         func = "feval", ncomp = neqn,parms=parms, rpar=rpar, ipar=ipar,islin=linear,
                                         initfunc="initrparms",bound = "bceval", jacfunc = "jeval", jacbound = "dbceval",
                                         dllname=problem,atol=rtol/1000,bspline=bspline,nmax=nmaxacc,...),
                                                                    
      )  
      solt <- solacc[,-1]  
    
    if (attributes(solacc)$name == "bvpcol") { 
      if (idid !=1) { 
        warning("bvpRtest : The mescd cannot be computed, the code has encountered trouble in the computation of the 
                accurate solution")
        mescd = NA 
        solt = NA
        runtime[3] = NA
        return(list(sol=sol,solt=solt,runTime=runTime[3],mescd=mescd))  
      }}
    else if (attributes(solacc)$name == "bvptwp") 
    { 
      if (idid > 0) { 
        warning("bvpRtest : The mescd cannot be computed, the code has encountered trouble in the computation of the 
               accurate solution")
        mescd = NA 
        solt = NA
        runTime[3] = NA
        return(list(sol=sol,solt=solt,runTime=runTime[3],mescd=mescd))  
      }}
    }
    
    
    dyn.unload(dllname)

    
    }
  
    
# end test problem is a fortran function
else{ 
# test problem is an R function
  probR <- problem()
  if (is.function(probR$prob)){ 
     outprob <-  probR$prob()

     fullnm <- outprob$fullnm
     problm <- outprob$problm
     typebvp <- outprob$typebvp
     neqn <- outprob$neqn
     ms <- outprob$ms
     nlbc <- outprob$nlbc
     aleft <- outprob$aleft
     aright <- outprob$aright
     numjac <- outprob$numjac
     numbcjac <- outprob$numbcjac
     linear <- outprob$linear
     Rpar <- outprob$rpar
     Ipar <- outprob$ipar}
  else{
    stop(sprintf("bvpRtest : no function named prob "))   
  }
  
  
  if (grep("SPBVP",typebvp) > 0) {
    if (is.null(eps)){
      eps <-  1e-1
    }
  }
  else if ( (solver == "acdc") || (solver == "acdcc") || (solver = "colmod")) {
    stop(sprintf("bvpRtest : A continuation solver cannot solve %s problems ",typebvp))
  }
  
  if (is.function(probR$setoutput)){
    outset <- probR$setoutput(neqn)
    solref <- outset$solref
    nindsol <-outset$nindsol
    indsol <- outset$indsol
  }
    else{
      stop(sprintf("bvpRtest : no function named setoutput "))   
    }
  if (is.function(probR$init)){ 
    outinit<-probR$init(neqn,ms,aleft,aright)
    givmsh <- outinit$givmsh
    givey <- outinit$givey
    nmsh <- outinit$nmsh
    xguess <- outinit$xguess        
    yguess <- outinit$yguess
    }
  else{
    stop(sprintf("bvpRtest : no function named init "))   
  }
  if (givmsh)
    x <- xguess
  else
    x <- seq(aleft,aright, by = (aright-aleft)/(nmsh-1) )
  
  
  
  if (!givey){
      xguess<-NULL
      yguess<-NULL
  }
  
  if (numjac) 
     jacfunc <- NULL
  else
     jacfunc <- probR$jeval
  
  if (numbcjac)
    jacbound <- NULL
  else
    jacbound <- probR$dbceval
  
  parms = c(EPS=eps)
  
  
  
  switch(solver,
         "twpbvp_l"=, "twpbvp_m"=,"acdc" = cond <- FALSE,
         "twpbvpc_l"=, "twpbvpc_m"= ,"acdcc"= cond <- TRUE
  )
  
  switch(solver,
         "twpbvp_l"=, "twpbvpc_l" = lobatto <- TRUE,
         "twpbvp_m"=, "twpbvpc_m" = lobatto <- FALSE
  )
  
  switch(solver, 
         "colmod"=,"colnew" = bspline <- FALSE,
         "colsys" = bspline <- TRUE)
  
  if (solver == "bvpshoot"){
    ya = rep(0,neqn)
    guess = rep(0,neqn)
    for ( i in 1:nlbc ) {  
       guess[i] = - probR$bceval(i, ya, eps,Rpar,Ipar)
    }
  }
  
   
  
  runTime<-system.time(switch(solver, 
         "twpbvp_l" =,  "twpbvpc_l" = ,  "twpbvp_m" =,  "twpbvpc_m" =
                              sol  <- bvptwp(x=x, leftbc = nlbc,order=ms, xguess=xguess,yguess=yguess,
                                      func = probR$feval, ncomp = neqn,parms=parms, 
                                      islin=linear,Rpar=Rpar,Ipar=Ipar,
                                      bound = probR$bceval, jacfunc = jacfunc, jacbound = jacbound,
                                      atol=tol,lobatto=lobatto,cond=cond,nmax=nmax,...),
           
          "acdc" =,  "acdcc" = sol  <- bvptwp(x=x, leftbc = nlbc,order=ms, xguess=xguess,yguess=yguess,
                                    eps=eps,epsini=Rpar[1],func = probR$feval, ncomp = neqn,parms=parms, 
                                    islin=linear,Rpar=Rpar,Ipar=Ipar,
                                    bound = probR$bceval, jacfunc = jacfunc, jacbound = jacbound,
                                    atol=tol,cond=cond,nmax=nmax,...),
           
           
          "colmod" = sol  <- bvpcol(x=x, leftbc = nlbc, order=ms,xguess=xguess,yguess=yguess,
                                     eps=eps,epsini=Rpar[1],func = probR$feval, ncomp = neqn,parms=parms, 
                                     islin=linear,Rpar=Rpar,Ipar=Ipar,
                                     bound = probR$bceval, jacfunc = jacfunc, jacbound = jacbound,
                                     atol=tol,nmax=nmax,...),
         
          "colnew" =,  "colsys" =
                        sol <- bvpcol(x=x, leftbc = nlbc,order=ms, xguess=xguess,yguess=yguess,
                                     func = probR$feval, ncomp = neqn,parms=parms, 
                                     islin=linear,Rpar=Rpar,Ipar=Ipar,
                                     bound = probR$bceval, jacfunc = jacfunc, jacbound = jacbound,
                                     atol=tol,bspline=bspline,nmax=nmax,...),
          "bvpshoot" = sol  <- bvpshoot(x=x, leftbc = nlbc, order=ms,
                                     func = probR$feval, ncomp = neqn,parms=parms, 
                                     Rpar=Rpar,Ipar=Ipar,guess=guess,
                                     bound = probR$bceval, jacfunc = jacfunc, jacbound = jacbound,
                                     atol=tol,...),
                             
         
    ) )
  
  idid <- attributes(sol)$istate[1]
  

  if (attributes(sol)$name == "bvpcol") { 
    if (idid !=1) { 
      warning("bvpRtest : the code has encountered trouble in the computation of the 
              solution")
      mescd = NA 
      solt = NA
      runTime[3] = NA
      return(list(sol=sol,solt=solt,runTime=runTime[3],mescd=mescd))  
    }}
  else if (attributes(sol)$name == "bvptwp") 
  { 
    if (idid > 0) { 
      warning("bvpRtest : the code has encountered trouble in the computation of the 
              solution")
      mescd = NA 
      solt = NA
      runTime[3] = NA
      return(list(sol=sol,solt=solt,runTime=runTime[3],mescd=mescd))  
    }}
   
  
    if ( (solver == "colnew") | (solver == "colmod") | (solver == "colsys")){
      ninter = attributes(sol)$istate[7]
      x<- attributes(sol)$rstate[1:(ninter+1)] 
      print("intervallo")
      print(ninter)
#      x<- seq(aleft,aright,by=(aright-aleft)/(1000))
      solp <- matrix(nrow=ninter+1,ncol=neqn+1)
      solp <- approx(sol,x)    }
    else
     { x = sol[,1]
      solp <- sol}
    
   
  
    if (is.function(probR$esolu) && solref)
       {solt <- probR$esolu(x,parms=parms,Rpar=Rpar,Ipar=Ipar)
      }
    else{ 
      
      xguess <- solp[,1]
      yguess <- t(solp[,-1])
      nmaxacc= max(length(xguess)*2,nmax) 
    
      
    switch(solver, 
             "twpbvp_l" =,  "twpbvpc_l" = ,  "twpbvp_m" = ,  "twpbvpc_m" = 
                          solacc  <- bvptwp(x=x, leftbc = nlbc,order=ms, xguess=xguess,yguess=yguess,
                                          func = probR$feval, ncomp = neqn,parms=parms, 
                                          islin=linear,Rpar=Rpar,Ipar=Ipar,
                                          bound = probR$bceval, jacfunc = jacfunc, jacbound = jacbound,
                                          atol=tol/1000,lobatto=lobatto,cond=cond,allpoint=FALSE,nmax=nmaxacc,...),
                
             "acdc" =,  "acdcc" =  solacc <- bvptwp(x=x, leftbc = nlbc, order=ms,xguess=xguess,yguess=yguess,
                                      eps=eps,epsini=Rpar[1],func = probR$feval, ncomp = neqn,parms=parms, 
                                      islin=linear,Rpar=Rpar,Ipar=Ipar,
                                      bound = probR$bceval, jacfunc = jacfunc, jacbound = jacbound,
                                      atol=tol/1000,cond=cond,allpoint=FALSE,nmax=nmaxacc,...),
             
                
             "colmod" = solacc  <- bvpcol(x=x, leftbc = nlbc, order=ms,xguess=xguess,yguess=yguess,
                                       eps=eps,epsini=Rpar[1],func = probR$feval, ncomp = neqn,parms=parms, 
                                       islin=linear,Rpar=Rpar,Ipar=Ipar,
                                       bound = probR$bceval, jacfunc = jacfunc, jacbound = jacbound,
                                       atol=tol/1000,nmax=nmaxacc,...),
             
             "colnew" =,  "colsys" = solacc  <- bvpcol(x=x, leftbc = nlbc,order=ms, xguess=xguess,yguess=yguess,
                                       func = probR$feval, ncomp = neqn,parms=parms, 
                                       islin=linear,Rpar=Rpar,Ipar=Ipar,
                                       bound = probR$bceval, jacfunc = jacfunc, jacbound = jacbound,
                                       atol=tol/1000,bspline=bspline,nmax=nmaxacc,...),
                 
           
           "bvpshoot" = solacc  <- bvpshoot(x=x, leftbc = nlbc, order=ms,
                                        func = probR$feval, ncomp = neqn,parms=parms, 
                                        Rpar=Rpar,Ipar=Ipar,guess=guess,
                                        bound = probR$bceval, jacfunc = jacfunc, jacbound = jacbound,
                                        atol=tol/1000,...),        
           
           ) 
      
      idid <- attributes(solacc)$istate[1]
      
      if (attributes(solacc)$name == "bvpcol") { 
         
        if (idid !=1) { 
        warning("bvpRtest : The mescd cannot be computed, the code has encountered trouble in the computation of the 
               accurate solution")
        mescd = NULL 
        solt = NULL
        return(list(sol=sol,solt=solt,runTime=runTime[3],mescd=mescd))  
        }}
       else if (attributes(solacc)$name == "bvptwp") 
       { 
         if (idid > 0) { 
           warning("bvpRtest : The mescd cannot be computed, the code has encountered trouble in the computation of the 
               accurate solution")
           mescd = NULL 
           solt = NULL
           return(list(sol=sol,solt=solt,runTime=runTime[3],mescd=mescd))  
         }}
        
      
      solt=solacc[,-1]
     
    }
      
  
  
}
  
  mescd <- -log10(max(abs(solp[, -1] - solt) / (1 + abs(solt)) ))
  abserr <- max(abs(solp[, 2] - solt[,1]))
  
  
  if (printall){
  
  for (i in indsol+1){
    
    ylabel=sprintf("y(%g)",i-1)
    mescdi <- -log10(max(abs(solp[, i] - solt[,i-1]) / (1 + abs(solt[,i-1])) ))
    mes = sprintf("mescd(%g)",i-1)
    plot(solp[,1],solp[,i], xlab = "x", ylab = ylabel ,col="darkblue", type = "l") #pch=number for a different shape, type = "l" for line
    plot(solp[,1],solp[,i]-solt[,i-1], xlab = "x", ylab = ylabel ,col="darkblue", type = "l") #pch=number for a different shape, type = "l" for line
    
    title(sprintf('PROBLEM= %s  %s SOLVER = %s %s %s = %.2f',problm,' ',solver,'   ',mes,  mescdi),
          font.main=4,cex.main=0.8)#or main = title inserts title into the plot(...)     
  }
  
  
  cat("\n--------------------\n")
  cat('Test Set for BVP Solvers (release 0.5)\n')
  cat(paste('Solving',fullnm, 'using', solver))
  cat("\n")
  if (grep("SPBVP",typebvp) > 0){
    cat(paste('eps = ',eps)) 
  }
  
  diagnostics(sol)     
  
  cat(paste("using mixed error yields mescd = ",mescd) )
  cat("\n")
  cat(paste("maximum absolute  error yields = ",abserr) )
  cat("\n")
  cat(paste("CPU-time used : ", runTime[3]))
  cat("\n")
  
  }
  
  return(list(sol=sol,solt=solt,runTime=runTime[3],mescd=mescd,abserr=abserr))  
  
  
}
                  